
/***************************************************************************\
*  Name    : DDS_Sweeper                                                *
*  Author  : Beric Dunn (K6BEZ)                                             *
*  Notice  : Copyright (c) 2013  CC-BY-SA                                   *
*          : Creative Commons Attribution-ShareAlike 3.0 Unported License   *
*  Date    : 11/07/2016                                                     *
*  Version : 1.003                                                          *
*  Notes   : Written using for the Arduino Micro                            *
*          :   Pins:                                                        *
*          :   A0 - Reverse Detector Analog in                              *
*          :   A1 - Forward Detector Analog in                              *
*          : Modified by Norbert Redeker (DG7EAO) 07/2014                   *
*          : TFT Display mit ILI9341 Chip, SPI, 240 x 320                   *
*          : usglib Grafik Bibliothek   https://code.google.com/p/ucglib/   *
*          :                                                                *
*          : Modified by Len Bayles (KA7FTP) and Jon Titus (KZ1G) 10/2016   *
*          :  D4/D5 - Rotary encoder                                        *
*          :  Added band/frequency seup menu                                *
*          :  To change band or frequency selections edit data in the       *
*          :  f_data[] structure. The f_options decleration details         *
*          :  the fields.  Extensive comments added by Jon.                 *
\***************************************************************************/

#include <SPI.h>              // Needed include files must be in your working folder
#include "Ucglib.h"
#include <PinChangeInt.h>
#include <AdaEncoder.h>


char  *version = "V1.003 Nov 07, 2016 15:33";

// Define as constants Data signals used to control AD9850 DDS module
// Numbers refer to the Data signals LABELED, D1, D2, etc., on Arduino Pro Mini, NOT to
// electrical pin numbers on the Mini module!!!  Comments refer to electrical pin numbers
const int FQ_UD=9;    // Frequency update on rising edge of clock   //D9,  module pin 12
const int SDAT=11;    // Serial data to AD9850, 40 bits (5 bytes)   //D11, module pin 14
const int SCLK=10;    // Serial clock input to AD9850               //D10, module pin 13
const int RESET=12;   // Reset for AD9850, resets on logic-1        //D12. module pin 15

// Variables for the display
double vswrArray[110];        // Array for SWR values; max of 110
int z = 0;                    // Index for vswrArray
double SwrFreq = 14;          // Variable for frequency with SWR minimum
double SwrMin = 100;          // Variable for the minimum SWR value
double Freq1 = 1;             // Frequency link for bottom of display
double Freq2 = 15;            // Frequency for middle of bottom line of display
double Freq3 = 30;            // Freq for middle of bottom line on display
unsigned long milliold = 0;   // Millisecond value for debouncing interrupt (old)
unsigned long millinew = 0;   // Millisecond value for debouncing interrupt (new)
int sweep = 0;                 // Is set to 1 upon interrupt, in void Loop "perform_sweep"
double counter = 0;           // Counter to ignore first interrupt

// Variables for measurements
double Fstart_MHz = 1;        // Start Frequency for sweep
double Fstop_MHz = 30;        // Stop Frequency for sweep
double current_freq_MHz;      // Temp variable used during sweep
long serial_input_number;     // Used to build number from serial stream
int num_steps = 100;          // Number of steps to use in the sweep
char incoming_char;           // Character read from serial stream


//Constructor for the display.  Requires "Ucglib.h" file.

Ucglib_ILI9341_18x240x320_SWSPI ucg(/*sclk=*/ 10, /*data=*/ 11, /*cd=*/ 6 , /*cs=*/ 5, /*reset=*/ 4);


// Variables added for frequency setup via the rotary encoder
#define PINA 5    //Signal D5 (pin 8)
#define PINB 4    //Signal D4 (pin 7)

#define  MAX_SELECT      13  // Number of items in setup menu

volatile int setupMode = 0;  // Currently selected option from f_options
int  f_select  = 12;

// Structure declaration for each frequency band to scan and display in setup menu
struct  f_options  {
                    char  *band;          // Text to display in setup menu
                    int    p_x;           // Display X position
                    int    p_y;           // Display Y position
                    int    box_width;     // Box pixel width
                    double  Fstart_MHz;   // Sweep start frequency in Mhz
                    double  Fstop_MHz;    // Sweep stop frequency in Mhz
                    int     num_steps;    // Frequency sweep steps
                    double  Freq1;        // Display low frequency
                    double  Freq2;        // Display mid frequency
                    double  Freq3;        // Display high frequency
                    };

//Structure initialization for 10 ham bands and three frequency spans
struct  f_options  f_data[]  =
  {
    {"160M",60,28,60,1.8,2.0,102,1.8,1.9,2.0},
    {"80M",60,56,60,3.5,4.0,102,3.5,3.75,4.0},
    {"60M",60,84,60,5.3,5.5,102,5.3,5.4,5.6},
    {"40M",60,112,60,7,7.3,102,7,7.15,7.3},
    {"30M",60,140,60,10,10.2,102,10,10.1,10.2},
                                  
    {"20M",220,28,60,14,14.35,102,14,14.175,14.35},
    {"17M",220,56,60,18,18.2,102,18,18.1,18.2},
    {"15M",220,84,60,21,21.5,102,21,21.25,21.5},
    {"12M",220,112,60,24.8,25,102,24.8,24.9,25},                                  
    {"10M",220,140,60,28,30,102,28,29,30},
                                   
    {"1Mhz - 15Mhz",100,168,140,1,15,102,1,8,15},
    {"15Mhz - 30Mhz",100,196,140,15,30,102,15,22.5,30},
    {"1Mhz - 30Mhz",100,224,140,1,30,102,1,15,30}                                 
  };

//********** Start of Setup routine **********
// This setup routine runs only once when you press the reset button or apply power to the analyzer
void setup() {
  
  // Write information for the display
  ucg.begin(UCG_FONT_MODE_SOLID);   //Select font
  ucg.clearScreen();
  
  ucg.setRotate270();
  ucg.setFont(ucg_font_ncenR14r);
  ucg.setColor(0, 255, 0);          // Green
  //ucg.setColor(1, 255, 0, 0);     // Red, Index1
  //ucg.setColor(255, 255, 255);    // White
  //ucg.setColor(255, 0, 0);        // Red
  
  ucg.setPrintPos(0,75);            // Position cursor
  ucg.print("Arduino");             // Display "Arduino" on display
  ucg.setPrintPos(10,100);          // Same format for other messages...
  ucg.print("Antenna");
  ucg.setPrintPos(20,125);
  ucg.print("Analyzer");
  ucg.setPrintPos(30,150);
  ucg.print("K6BEZ - DG7EAO - KA7FTP");
  
  ucg.setPrintPos(40,240);
  ucg.print(version);      // Print Software Version to screen
  
  // Configiure Dx signals on Arduino Pro Mini as outputs to control AD9850 DDS module
  pinMode(FQ_UD,OUTPUT);
  pinMode(SCLK,OUTPUT);
  pinMode(SDAT,OUTPUT);
  pinMode(RESET,OUTPUT);
  
  // Interrupt on data signal D2 (pin 5) for rotary encoder
  pinMode(2,OUTPUT);                    // Interrupt signal on signal D2 (pin 5) for Rotray Encoder button
  digitalWrite(2, HIGH);                // Force to a logic-1
  attachInterrupt(0, key2, LOW);    // Interrupt #0, attached routine named key2, detected on falling edge of signal
  unsigned long milliold = millis();    // Millis represents an internal processor millisecond counter
                                        // that starts when your program starts. See Arduino Reference
                                        // for more information about the millis operation    
  
  pinMode(3,OUTPUT);                    // Interrupt signal on signal D3 (pin 6) for Start pushbutton
  digitalWrite(3, HIGH);                // Force to a logic-1
  attachInterrupt(1, key3, LOW);    // Interrupt #1, attached routine named key3, detected on falling edge of signal
  //milliold = millis();
  
  // Configure LED pin for digital output   // ???Doesn't look like this pin gets used for anything???
  pinMode(13,OUTPUT);

  // Set signals A0 (pin 17) and A1 (pin 18) for analog inputs and use
  // internal reference voltage for the analog-to-digital converter (ADC)
  pinMode(A0,INPUT);
  pinMode(A1,INPUT);
  analogReference(INTERNAL);    // Use internal reference for the processor's ADC (INTERNAL1V1 or INTERNAL2V56 For 1284P)
  
  Serial.begin(57600);           // Initialize serial communication at 57,600 bau

  digitalWrite(RESET,HIGH);      // Reset the AD9850 DDS via a short logic-1 pulse on the RESET output D12 (pin 15)
  digitalWrite(RESET,LOW);
  
  serial_input_number=0;        // Initialise the incoming-serial-data number to zero

  // Set up our rotray encorder for 
  AdaEncoder::addEncoder('a', PINA, PINB);  // Set up our rotray encorder for signal D5 (pin 8), and signal D4 (pin 7)
}
// ********** end of Setup routine **********


// ********** Start of Serial Input routine that runs continuously *********
// This routine accepts serial information from a PC application program
// And displays SWR and other information.  Download the application from:
// http://???
// This code accepts numeric information (0 through 9) as well as command codes:
// A = Use keyed in value as the start-of-sweep frequency (Fstart_MHz)
// B = Use keyed in value as the end-of-sweep frequency (Fstop_MHz)
// C = Use the Fstart value to set the DDS output to produce only that frequency
// N = Set the number of frequency steps you want in a band sweep
// S or s = Perform a sweep between FStart and FStop
// ? = Send current configuration data to PC application

void loop() {
  
if(Serial.available()>0)              //Check for character
    {
      incoming_char = Serial.read();  //New character available, get it
      switch(incoming_char)
      {
        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
           // Shift latest digit to the left and add new digit
           serial_input_number=serial_input_number*10+(incoming_char-'0');
        break;
        
        case 'A':     //Convert input value into frequency FStart_MHz
           Fstart_MHz = ((double)serial_input_number)/1000000;
           serial_input_number=0;                               //Clear serial-input-number
        break;
        
        case 'B':     //Convert input value into frequency FStop_MHz
           Fstop_MHz = ((double)serial_input_number)/1000000;
           serial_input_number=0;                               //Clear serial-input-number
        break;
        
        case 'C':     // Convert input value to Fstart_MHz and set DDS output to this single frequency
           Fstart_MHz = ((double)serial_input_number)/1000000;  // Get only MHz part of the value
           //SetDDSFreq(Fstart_MHz);                            // 14090123 --> 14
           SetDDSFreq(Fstart_MHz * 1000000);                    // Get back to MHz: 14 -->14000000
           delay(100);                                          // Set the DDS frequency; short delay
           SetDDSFreq(Fstart_MHz * 1000000);                    // Set the DDS frequency again???
           serial_input_number=0;                               // Clear serial-input-number    
        break;
        
        case 'N':     // Use serial_input_number to set number of frequency steps in sweep
           num_steps = serial_input_number;
           serial_input_number=0;                   //Clear serial-input-number
        break;
        
        case 'S':     // On receiving "S" or "s" start a frequency sweep based on data already entered    
        case 's':    
           Perform_sweep();
         break;
         
        case '?':     // On receiving "?", report current configuration to PC    
           Serial.print("Start Freq:");
           Serial.println(Fstart_MHz*1000000);
           Serial.print("Stop Freq:");
           Serial.println(Fstop_MHz*1000000);
           Serial.print("Num Steps:");
           Serial.println(num_steps);
        break;
      }
    Serial.flush();   //Clear any remaining data from the serial buffer     
  } 
 // ********** End of Serial Input routine that runs continuously *********

  
 // ********** Start of Frequency-Sweep Overall routine **********
 // Perform frequency sweep after an interrupt at input D2 (pin5) or input D3 (pin 6)
 // Ignore Startup Interrupts by counter

 if (sweep == 1)             // If sweep equals 1 from pressing single pushbutton button, run Perform_sweep();
  {                                       
   sweep = 0;                              // Clear the flag; reset for another interrupt
     
   // The following five statements obtain information from the ten f_data structures defines earlier
   // The f-select value determines which structure to access. The comments refer to a selection of 80M
   Fstart_MHz = f_data[f_select].Fstart_MHz;  // Get the starting frequency: 3.5 (MHz)
   Fstop_MHz = f_data[f_select].Fstop_MHz;    // Get the stop frequency 4.0 (MHz)
   num_steps = f_data[f_select].num_steps;    // Get number of steps (102)
   Freq1 = f_data[f_select].Freq1;            // Get the left-most frequency to display on the SWR graph (3.5)   
   Freq2 = f_data[f_select].Freq2;            // Get the middle frequency to display on the SWR graph (3.75)
   Freq3 = f_data[f_select].Freq3;            // Get the right-most frequency to display on the SWR graph (4.0)
   
   Perform_sweep();                           //Go to the Perform_sweep function and run an SWR scan 
  }
  

  if(setupMode == 1)           //If setupMode equals 1 from pressing rotrary encoder button, run the setupMenu function
     {                                  
     setupMenu();
     }
}                                             // If setupMode not equal to 1, continue here 
// ********** End of Frequency Sweep  **********


// **********Start of Perform_sweep function **********
void Perform_sweep()
{
  double FWD=0;               //Define and initialize variables for forward and reverse voltages, and VSWR 
  double REV=0;
  double DENOM;
  double VSWR;
  double Fstep_MHz = (Fstop_MHz-Fstart_MHz)/num_steps;  //Calculate the frequency "jump" for the given number of steps
  
  z = 0;                      // Initialize array index to 0
  SwrMin = 100;               // Set SwrMin to a high number to start. Calculated VSWRs will be lower.
  
  ucg.clearScreen();          // Display the "Scanning..." message on the display
  ucg.setPrintPos(150,150);
  ucg.print("Scanning..."); 
   
  // Start loop to make a VSWR measurement for each frequency step, including first and last frequency in the span 
  for(int i=0; i<=num_steps; i++)
    {
    current_freq_MHz = Fstart_MHz + (i*Fstep_MHz);  // Calculate frequency to use for a measurement
    SetDDSFreq(current_freq_MHz*1000000);           // Set DDS to calculated frequency
    delay(100);                                     // Short delay for DDS to settle
    REV = analogRead(A0);                           // Read the forward and reverse voltages from the MCU's ADC
    FWD = analogRead(A1);
    
    REV = REV-5;                    // Offset Correction.  WHAT IS THIS FOR?  ADC INACCURACY OR DC OFFSET?
        
    //if(REV>=FWD){REV = FWD-1;}      //????
    
    if (REV <1)           // Prevents REV from ever equaling 0
      REV = 1;
    
    DENOM = (FWD-REV);         // do the subtraction here
    DENOM = abs(DENOM);        // get absolute value here.  See doc for abs limitations.  Shouldn't
                               // do math within an abs statement:  NO! -->  DENOM = abs(FWD-REV);
                           
    VSWR = (FWD + REV)/DENOM;  // Calculate VSWR    
    
    VSWR = VSWR * 1000;             // Scale VSWR value for output  ???DO THIS IN Serial.print statement???
    
    // Send this information to PC over serial connection
    Serial.print(current_freq_MHz*1000000);
    Serial.print(",0,");
    Serial.print(VSWR);     //DO MULT BY 1000 HERE INSTEAD???  SEE BELOW.
    Serial.print(",");
    Serial.print(FWD);
    Serial.print(",");
    Serial.println(REV);
    
    // Put the VSWR value in SWR an Array and get frequency at lowest VSWR value
    vswrArray[z] = VSWR/1000;                       // ???NOW DIVIDE BY 1000.  UNNEEDED IF MULT IN PRINT STATEMENT???
    if (vswrArray[z] > 10) vswrArray[z] = 10;       // Limit maximum VSWR to 10
    if (vswrArray[z] < SwrMin && vswrArray[z] > 1)  //??? 
    {
      SwrMin = vswrArray[z];                        // Put 
      SwrFreq = current_freq_MHz; 
    }    
    z = z + 1;                                      // Increment z for next point in vswrArray  
  }                                                 // VSWR scan ends here
  
  Serial.println("End");                            // Send "End" to PC to indicate end of sweep
  Serial.flush();                                   // Clean out any data from the serial input port 
  
  ucg.clearScreen();                                // Clear the LCD area
  CreateGrid();                                     // Create the grid
 
  ucg.setColor(255, 255, 0);                        // Set line color to yellow
    
  // Draw Line
  
  // 30 = swr 10    210 = swr 0
  // Diff swr 10 = 180
  // swr 2 = 18 * 2
  
double last = 10;
double xx = 6;
int j = 1;

for (int i = 1; i < 103; i++)                       // Loop to display VSWR plot of 102 points on the grid
  {
  xx = vswrArray[i];                                // Get the next VSWR value
  
  ucg.drawLine(j,210-last*18, j+1, 210-xx*18);      // Draw line between previous VSWR and next VSWR values???
  ucg.drawLine(j+1,210-last*18, j+2, 210-xx*18);
  j = j + 3;                                        // Increase j by 3 for the enxt set of VSWR points
last = xx;  
  }     
}
// **********End of Perform_sweep function **********


// **********Start of SetDDSFreq function **********
// This routine controls the settings in the AD9850 DDS chip to
// produce the RF signal used to measure forward and backward voltages
// See the Analog Devices AD9850 data sheet for programming details

void SetDDSFreq(double Freq_Hz)
  {
    int32_t f = Freq_Hz * 4294967295/125000000;       // Calculate the DDS word (see AD9850 Datasheet)
    for (int b=0; b<4; b++,f>>=8)                     // Send one byte at a time
        {
          send_byte(f & 0xFF);
        }
    send_byte(0);                                     // 5th byte must equal 0
    digitalWrite(FQ_UD,HIGH);                         // Logic-1 pulse to load values to AD9850
    digitalWrite(FQ_UD,LOW);
  }
// ********** End of SetDDSFreq function **********


// ********** Start of send_byte function **********
// Break bytes into bits for transmission via an SPI interface
void send_byte(byte data_to_send)
  {
    for (int i=0; i<8; i++,data_to_send>>=1)      // Go through loop 8 times, once per bit
      {
        digitalWrite(SDAT,data_to_send & 0x01);   // Put the bit on output signal D11 (pin 14)
        digitalWrite(SCLK,HIGH);                  // Logic-1 pulse to AD9850 to accept serial bit
        digitalWrite(SCLK,LOW);                   // pulse output on signal D10 (pin 13)
      }
  }
// ********** End of send_byte function **********  


// ********** Start of CreateGrid function **********  
void CreateGrid()
  {
  //ucg.clearScreen();
  double maxSwr = 10;           // Preset maximum SWR to display at 10

  ucg.drawHLine(0,120,310);     // Draw horizontal and vertical lines on the LCD
  ucg.drawHLine(0,196,310);
       
  ucg.drawVLine(78,30,180);
  ucg.drawVLine(155,30,180);
  ucg.drawVLine(233,30,180);
        
	ucg.setPrintPos(0, 235);      // Put three frequency values on the LCD
	ucg.print(Freq1,3);
 
	ucg.setPrintPos(130, 235);
	ucg.print(Freq2,3);
 
	ucg.setPrintPos(260, 235);
	ucg.print(Freq3,3);
 
	ucg.setPrintPos(10, 15);      // Put "SWR" on LCD
	ucg.print("SWR");
 
	ucg.setPrintPos(70, 15);      // Put minimum SWR value on LCD
	ucg.print(SwrMin,2);

 ucg.setPrintPos(115, 15);
	ucg.print(">");
 
	ucg.setPrintPos(130, 15);     // Put maximum SWR value on LCD
	ucg.print(maxSwr,2);

  ucg.setPrintPos(250, 15);
	//ucg.print((freqCenter/1000000*1.05),3);
  ucg.print(SwrFreq,3);
        
	ucg.drawRFrame(0,30,310,180, 1);
  }  
// ********** End of CreateGrid function **********


// ********** Start of Interrupt Service Routine for Button on Rotary Encoder **********
// Interrupt on signal D2 (pin 5)

void key2()
{
   counter = counter + 1;           // Ignore Startup Interrupts > counter
   millinew = millis();             // Get millisecond (msec) value from processor and save at millinew
                                    // Processor millisecond count starts when your program starts 
   if (millinew - milliold < 1000)  // Subtract old msec count from new msec count. If less than 1000 msec (1 sec)
     {                              // make milliold equal to millinew.  This is basically a 1-second delay to 
        milliold = millinew;        // debounce the rotary encoder pushbutton switch contacts.
        return;                     // Then teturn. 
     }
   milliold = millinew;
  
    setupMode = 1;
  }
// ********** End of Interrupt Service Routine for Button on Rotary Encoder **********


// ********** Start of Interrupt Service Routine for Sweep Button **********
// Interrupt on signal D3 (pin 6)

void key3()
  {
    millinew = millis();            
    if (millinew - milliold < 1000)   // Steps to debounce pushbutton contact closure
      {                               // see comments for interrupt routine key3
        milliold = millinew;
        return; 
      }
    milliold = millinew;
 
      sweep = 1;
      setupMode = 0;
  }
// ********** End of Interrupt Service Routine for Sewwp Button ********** 


// ********** Start of Function to Display Startup Menu and Use Rotary Encoder **********
void setupMenu()
  {
    int  s_update = 1, last = 0;
    encoder *myEncoder;
    
    setupScreen(); 
    
    while(setupMode == 1)
      {
        if(s_update)
          {
            ucg.setColor(0, 0, 0); //Black
            ucg.drawFrame(f_data[last].p_x -5, f_data[last].p_y -20, f_data[last].box_width, 25);
          
            ucg.setColor(255, 255, 255); //White
            ucg.drawFrame(f_data[f_select].p_x -5, f_data[f_select].p_y -20, f_data[f_select].box_width, 25);
          
            s_update = 0;
          }
       myEncoder=AdaEncoder::genie();
       if(myEncoder != NULL)
         {       
           last = f_select;
           if (myEncoder->clicks > 0)
             {
               if(f_select == MAX_SELECT -1)
                 f_select = 0;
               else
                 f_select++;
              
               s_update = 1;
             }
          else if(myEncoder->clicks < 0) 
            {
              if(f_select)
                --f_select;
              else
                f_select = MAX_SELECT -1;
              
              s_update = 1;
            }
          myEncoder->clicks = 0;
         }
     }
  // Update start and stop freq and display frequencies???
  ucg.clearScreen(); 
  }
 // ********** End of Function to Display Startup Menu and Use Rotary Encoder **********  

  
 // ********** Start of Function setupScreen **********
 void setupScreen()
   {
     int  i;
     ucg.clearScreen();
     ucg.setColor(255, 255, 255); //White
     for(i = 0 ; i < MAX_SELECT ; i++)
       {
         ucg.setPrintPos(f_data[i].p_x,f_data[i].p_y);
         ucg.print(f_data[i].band);      
      }
   }
// ********** End of Function setupScreen **********  

// ************************************
// ********** END OF PROGRAM **********
// ************************************
  
